%
%  Virtual Location List
%
%  This provides a virtual list of units at this place to move
%  through, not unlike the list provided by VPA in the lower-right
%  corner of the screen.
%
%  Imagine all units at this place written down in a list, like this:
%     Base
%     Planet
%     Ship A
%     Ship B
%     Ship C
%     ...
%  This module enables you to cycle through this list using the
%  [Up] and [Down] arrows.
%
%  Problem: in the starchart, up/down arrows will no longer work in
%  PCC 1.1.16 and below. PCC 1.1.17 changes keymap behavior so that
%  it now works as expected.
%
%  07/Jul/2008, Stefan Reuther
%

Print "[updown.q 07/Jul/2008]"

Sub MoveToHighestIdShipHere
  Local X := Loc.X, Y := Loc.Y
  Local i := FindShip(Loc.X=X And Loc.Y=Y And Owner$=My.Race$)
  Local j
  Do While j := FindShip(Id > i And Loc.X=X And Loc.Y=Y And Owner$=My.Race$)
    i := j
  Loop
  UI.GotoScreen 1, i
EndSub

Sub MoveUpHere
  Local i, j, k, X, Y
  If UI.Screen=2 And Base.YesNo Then
    % Planet with base
    % -> go to base
    UI.GotoScreen 3, Id
  Else If UI.Screen=2 Or UI.Screen=3 Then
    % I'm at a base
    % I'm at a planet without base
    % -> move to highest-Id ship
    MoveToHighestIdShipHere
  Else If UI.Screen=1 Then
    % I'm at a ship
    % -> move to lower Id
    j := Id
    X := Loc.X
    Y := Loc.Y
    i := FindShip(Id < j And X=Loc.X And Y=Loc.Y And Owner$=My.Race$)
    Do While k := FindShip(Id > i And Id < j And Loc.X=X And Loc.Y=Y And Owner$=My.Race$)
      i := k
    Loop
    If i Then
      UI.GotoScreen 1, i
    Else
      If Orbit$ And Planet(Orbit$).Owner$=My.Race$ Then
        % No lower-Id ship, so pick planet
        UI.GotoScreen 2, Orbit$
      Else
        % No lower-Id ship and no planet, so perform wraparound
        MoveToHighestIdShipHere
      EndIf
    EndIf
  EndIf
EndSub

Sub MoveDownHere
  Local i, j, X, Y
  If UI.Screen=3 Then
    % Base
    % -> go to planet
    UI.GotoScreen 2, Id
  Else If UI.Screen=2 Then
    % Planet
    % -> go to first ship
    i := FindShip(Orbit$=Planet.Id And Owner$=Planet.Owner$)
    If i Then
      UI.GotoScreen 1, i
    Else 
      If Base.YesNo Then UI.GotoScreen 3, id
    EndIf
  Else If UI.Screen=1 Then
    % Ship
    % -> go to next higher Id ship
    j := Id
    X := Loc.X
    Y := Loc.Y
    i := FindShip(Id > j And Loc.X=X And Loc.Y=Y And Owner$=My.Race$)
    If i Then
      UI.GotoScreen 1, i
    Else
      If Orbit$ Then
        If Planet(Orbit$).Base.YesNo Then
          UI.GotoScreen 3, Orbit$
        Else
          UI.GotoScreen 2, Orbit$
        EndIf
      Else
        i := FindShip(Loc.X=X And Loc.Y=Y And Owner$=My.Race$)
        UI.GotoScreen 1, i
      EndIf
    EndIf
  EndIf
EndSub

Bind ShipScreen   "up" := "MoveUpHere", "down" := "MoveDownHere"
Bind PlanetScreen "up" := "MoveUpHere", "down" := "MoveDownHere"
Bind BaseScreen   "up" := "MoveUpHere", "down" := "MoveDownHere"
